@@ -13,7 +13,7 @@ namespace Tomboy
 		string notes_dir;
 		string backup_dir;
 		ArrayList notes;
-		PluginManager plugin_mgr;
+		AddinManager addin_mgr;
 		TrieController trie_controller;
 		
 		static string start_note_uri = String.Empty;
@@ -53,8 +53,8 @@ namespace Tomboy
 			bool first_run = FirstRun ();
 			CreateNotesDir ();
 
-			plugin_mgr = CreatePluginManager ();
 			trie_controller = CreateTrieController ();
+			addin_mgr = CreateAddinManager ();
 
 			if (first_run) {
 				// First run. Create "Start Here" notes.
@@ -66,32 +66,20 @@ namespace Tomboy
 			Tomboy.ExitingEvent += OnExitingEvent;
 		}
 
-		// Create & populate the Plugins dir. For overriding in test
-		// methods.
-		protected virtual PluginManager CreatePluginManager ()
-		{
-			// Allow developers to override the plugins path with an
-			// environment variable.
-			string plugins_dir =
-				Environment.GetEnvironmentVariable ("TOMBOY_PLUGINS_PATH");
-
-			if (plugins_dir == null) {
-				string tomboy_dir = 
-					Path.Combine (Environment.GetEnvironmentVariable ("HOME"), 
-						      ".tomboy");
-				plugins_dir = Path.Combine (tomboy_dir, "Plugins");
-			}
-
-			PluginManager.CreatePluginsDir (plugins_dir);
-
-			return new PluginManager (plugins_dir);
-		}
-
 		// Create the TrieController. For overriding in test methods.
 		protected virtual TrieController CreateTrieController ()
 		{
 			return new TrieController (this);
 		}
+		
+		protected virtual AddinManager CreateAddinManager ()
+		{
+			string tomboy_conf_dir =
+				Path.Combine (Environment.GetEnvironmentVariable ("HOME"),
+							".tomboy");
+			
+			return new AddinManager (tomboy_conf_dir);
+		}
 
 		// For overriding in test methods.
 		protected virtual bool DirectoryExists (string directory)
@@ -210,15 +198,27 @@ namespace Tomboy
 				}
 			}
 
-			// Update the trie so plugins can access it, if they want.
+			// Update the trie so addins can access it, if they want.
 			trie_controller.Update ();
+			
+			bool startup_notes_enabled = (bool)
+					Preferences.Get (Preferences.ENABLE_STARTUP_NOTES);
 
-			// Load all the plugins for our notes.
+			// Load all the addins for our notes.
 			// Iterating through copy of notes list, because list may be
-			// changed when loading plugins.
+			// changed when loading addins.
 			ArrayList notesCopy = new ArrayList (notes);
 			foreach (Note note in notesCopy) {
-				plugin_mgr.LoadPluginsForNote (note);
+				addin_mgr.LoadAddinsForNote (note);
+
+				// Show all notes that were visible when tomboy was shut down
+				if (note.IsOpenOnStartup) {
+					if (startup_notes_enabled)
+						note.Window.Show ();
+					
+					note.IsOpenOnStartup = false;
+					note.QueueSave (false);
+				}
 			}
 			
 			// Make sure that a Start Note Uri is set in the preferences.  This
@@ -234,9 +234,24 @@ namespace Tomboy
 
 		void OnExitingEvent (object sender, EventArgs args)
 		{
+			// Call ApplicationAddin.Shutdown () on all the known ApplicationAddins
+			foreach (ApplicationAddin addin in addin_mgr.GetApplicationAddins ()) {
+				try {
+					addin.Shutdown ();
+				} catch (Exception e) {
+					Logger.Warn ("Error calling {0}.Shutdown (): {1}",
+							addin.GetType ().ToString (), e.Message);
+				}
+			}
+
 			Logger.Log ("Saving unsaved notes...");
 
 			foreach (Note note in notes) {
+				// If the note is visible, it will be shown automatically on
+				// next startup
+				if (note.HasWindow && note.Window.Visible)
+					note.IsOpenOnStartup = true;
+				
 				note.Save ();
 			}
 		}
@@ -337,12 +352,24 @@ namespace Tomboy
 
 			Note new_note = Create (title, content);
 
-			// Select the inital "Describe..." text so typing will
-			// immediately overwrite...
-			NoteBuffer buffer = new_note.Buffer;
-			Gtk.TextIter iter = buffer.GetIterAtOffset (header.Length);
-			buffer.MoveMark (buffer.SelectionBound, iter);
-			buffer.MoveMark (buffer.InsertMark, buffer.EndIter);
+			// In case the note was created using the "Link" button, Select the inital 
+         		// "Describe..." text so typing will overwrite the body text, 
+			// otherwise select the title 
+			NoteBuffer buffer = new_note.Buffer;	
+         		Gtk.TextIter iter;
+			bool linkedNote = !title.StartsWith (Catalog.GetString ("New Note"));
+			
+         		if (linkedNote)
+            			iter = buffer.GetIterAtOffset (header.Length);
+			else
+				iter = buffer.GetIterAtOffset (0);
+
+         		buffer.MoveMark (buffer.SelectionBound, iter);
+
+         		if (linkedNote)
+				buffer.MoveMark (buffer.InsertMark, buffer.EndIter);
+		   	else
+				buffer.MoveMark (buffer.InsertMark, buffer.GetIterAtOffset (title.Length));
 
 			return new_note;
 		}
@@ -365,8 +392,8 @@ namespace Tomboy
 
 			notes.Add (new_note);
 
-			// Load all the plugins for the new note
-			plugin_mgr.LoadPluginsForNote (new_note);
+			// Load all the addins for the new note
+			addin_mgr.LoadAddinsForNote (new_note);
 
 			if (NoteAdded != null)
 				NoteAdded (this, new_note);
@@ -423,18 +450,19 @@ namespace Tomboy
 			}
 		}
 
-		public PluginManager PluginManager
-		{
-			get {
-				return plugin_mgr;
-			}
-		}
-
 		public TrieTree TitleTrie
 		{
-			get {
-				return trie_controller.TitleTrie;
-			}
+			get { return trie_controller.TitleTrie; }
+		}
+		
+		public AddinManager AddinManager
+		{
+			get { return addin_mgr; }
+		}
+		
+		public string NoteDirectoryPath
+		{
+			get { return notes_dir; }
 		}
 
 		public event NotesChangedHandler NoteDeleted;
