@@ -1,6 +1,7 @@
 
 using System;
 using System.IO;
+using System.Xml;
 using Mono.Unix;
 
 namespace Tomboy 
@@ -11,6 +12,7 @@ namespace Tomboy
 		static TomboyTrayIcon tray_icon;
 		static bool tray_icon_showing = false;
 		static bool is_panel_applet = false;
+		static PreferencesDialog prefs_dlg;
 #if ENABLE_DBUS
 		static RemoteControl remote_control;
 #endif
@@ -32,7 +34,7 @@ namespace Tomboy
 
 			Initialize ("tomboy", "Tomboy", "tomboy", args);
 
-			PluginManager.CheckPluginUnloading = cmd_line.CheckPluginUnloading;
+//			PluginManager.CheckPluginUnloading = cmd_line.CheckPluginUnloading;
 
 			// Create the default note manager instance.
 			string note_path = GetNotePath (cmd_line.NotePath);
@@ -49,6 +51,12 @@ namespace Tomboy
 			}
 #endif
 			ActionManager am = Tomboy.ActionManager;
+			
+			ApplicationAddin [] addins =
+				manager.AddinManager.GetApplicationAddins ();
+			foreach (ApplicationAddin addin in addins) {
+				addin.Initialize ();
+			}
 
 			if (cmd_line.UsePanelApplet) {
 				tray_icon_showing = true;
@@ -195,12 +203,25 @@ namespace Tomboy
 		
 		static void OnShowPreferencesAction (object sender, EventArgs args)
 		{
-			tray_icon.TomboyTray.ShowPreferences ();
+			if (prefs_dlg == null) {
+				prefs_dlg = new PreferencesDialog (manager.AddinManager);
+				prefs_dlg.Response += OnPreferencesResponse;
+			}
+			prefs_dlg.Present ();
+		}
+
+		static void OnPreferencesResponse (object sender, Gtk.ResponseArgs args)
+		{
+			((Gtk.Widget) sender).Destroy ();
+			prefs_dlg = null;
 		}
 		
 		static void OnShowHelpAction (object sender, EventArgs args)
 		{
-			GuiUtils.ShowHelp("tomboy.xml", null, tray_icon.TomboyTray.Screen, null);
+			// Pass in null for the screen when we're running as a panel applet
+			GuiUtils.ShowHelp("tomboy.xml", null,
+					tray_icon == null ? null : tray_icon.TomboyTray.Screen,
+					null);
 		}
 		
 		static void OnShowAboutAction (object sender, EventArgs args)
@@ -208,6 +229,7 @@ namespace Tomboy
 			string [] authors = new string [] {
 				"Alex Graveley <alex@beatniksoftware.com>",
 				"Boyd Timothy <btimothy@gmail.com>",
+				"Chris Scobell <chris@thescobells.com>",
 				"David Trowbridge <trowbrds@gmail.com>",
 				"Ryan Lortie <desrt@desrt.ca>",
 				"Sandy Armstrong <sanfordarmstrong@gmail.com>",
@@ -230,8 +252,8 @@ namespace Tomboy
 				Catalog.GetString ("Copyright \xa9 2004-2007 Alex Graveley");
 			about.Comments = Catalog.GetString ("A simple and easy to use desktop " +
 							    "note-taking application.");
-			about.Website = "http://www.beatniksoftware.com/tomboy";
-			about.WebsiteLabel = Catalog.GetString("Homepage & Donations");
+			about.Website = Defines.TOMBOY_WEBSITE;
+			about.WebsiteLabel = Catalog.GetString("Homepage");
 			about.Authors = authors;
 			about.Documenters = documenters;
 			about.TranslatorCredits = translators;
@@ -274,11 +296,12 @@ namespace Tomboy
 		bool open_start_here;
 		string open_note_uri;
 		string open_note_name;
+		string open_external_note_path;
 		string highlight_search;
 		string note_path;
 		string search_text;
 		bool open_search;
-		bool check_plugin_unloading;
+//		bool check_plugin_unloading;
 
 		public TomboyCommandLine (string [] args)
 		{
@@ -297,7 +320,8 @@ namespace Tomboy
 						open_note_name != null ||
 						open_note_uri != null || 
 						open_search ||
-						open_start_here;
+						open_start_here ||
+						open_external_note_path != null;
 			}
 		}
 
@@ -306,10 +330,10 @@ namespace Tomboy
 			get { return note_path; }
 		}
 
-		public bool CheckPluginUnloading
-		{
-			get { return check_plugin_unloading; }
-		}
+//		public bool CheckPluginUnloading
+//		{
+//			get { return check_plugin_unloading; }
+//		}
 
 		public static void PrintAbout () 
 		{
@@ -348,10 +372,11 @@ namespace Tomboy
 					"in the opened note.\n");
 #endif
 
-			usage +=
-				Catalog.GetString (
-					"  --check-plugin-unloading\tCheck if plugins are " +
-					"unloaded properly.\n");
+// TODO: Restore this functionality with addins
+//			usage +=
+//				Catalog.GetString (
+//					"  --check-plugin-unloading\tCheck if plugins are " +
+//					"unloaded properly.\n");
 
 #if !ENABLE_DBUS
 			usage += Catalog.GetString ("D-BUS remote control disabled.\n");
@@ -399,7 +424,10 @@ namespace Tomboy
 					// If the argument looks like a Uri, treat it like a Uri.
 					if (args [idx].StartsWith ("note://tomboy/"))
 						open_note_uri = args [idx];
-					else
+					else if (File.Exists (args [idx])) {
+						// This is potentially a note file
+						open_external_note_path = args [idx];
+					} else
 						open_note_name = args [idx];
 
 					break;
@@ -475,9 +503,9 @@ namespace Tomboy
 					open_search = true;
 					break;
 
-				case "--check-plugin-unloading":
-					check_plugin_unloading = true;
-					break;
+//				case "--check-plugin-unloading":
+//					check_plugin_unloading = true;
+//					break;
 
 				case "--version":
 					PrintAbout ();
@@ -544,6 +572,52 @@ namespace Tomboy
 					remote.DisplayNote (open_note_uri);
 			}
 			
+			if (open_external_note_path != null) {
+				string note_id = Path.GetFileNameWithoutExtension (open_external_note_path);
+				if (note_id != null && note_id != string.Empty) {
+					// Attempt to load the note, assuming it might already
+					// be part of our notes list.
+					if (remote.DisplayNote (
+							string.Format ("note://tomboy/{0}", note_id)) == false) {
+
+						StreamReader sr = File.OpenText (open_external_note_path);
+						if (sr != null) {
+							string noteTitle = null;
+							string noteXml = sr.ReadToEnd ();
+							
+							// Make sure noteXml is parseable
+							XmlDocument xmlDoc = new XmlDocument ();
+							try {
+								xmlDoc.LoadXml (noteXml);
+							} catch {
+								noteXml = null;
+							}
+							
+							if (noteXml != null) {
+								noteTitle = NoteArchiver.Instance.GetTitleFromNoteXml (noteXml);
+								if (noteTitle != null) {
+									// Check for conflicting titles
+									string baseTitle = (string)noteTitle.Clone ();
+									for (int i = 1; remote.FindNote (noteTitle) != string.Empty; i++)
+										noteTitle = baseTitle + " (" + i.ToString() + ")";
+									
+									string note_uri = remote.CreateNamedNote (noteTitle);
+									
+									// Update title in the note XML
+									noteXml = NoteArchiver.Instance.GetRenamedNoteXml (noteXml, baseTitle, noteTitle);
+
+									if (note_uri != null) {
+										// Load in the XML contents of the note file
+										if (remote.SetNoteContentsCompleteXml (note_uri, noteXml))
+											remote.DisplayNote (note_uri);
+									}
+								}
+							}
+						}
+					}
+				}
+			}
+			
 			if (open_search) {
 				if (search_text != null)
 					remote.DisplaySearchWithText (search_text);
