@@ -8,17 +8,17 @@ using Mono.Unix;
 
 namespace Tomboy
 {
-	public class NoteRenameWatcher : NotePlugin
+	public class NoteRenameWatcher : NoteAddin
 	{
 		bool editing_title;
 		Gtk.TextTag title_tag;
 
-		protected override void Initialize ()
+		public override void Initialize ()
 		{
 			title_tag = Note.TagTable.Lookup ("note-title");
 		}
 
-		protected override void Shutdown ()
+		public override void Shutdown ()
 		{
 			// Do nothing.
 		}
@@ -37,7 +37,7 @@ namespace Tomboy
 			get { return Buffer.StartIter; }
 		}
 
-		protected override void OnNoteOpened ()
+		public override void OnNoteOpened ()
 		{
 			Buffer.MarkSet += OnMarkSet;
 			Buffer.InsertText += OnInsertText;
@@ -186,11 +186,14 @@ namespace Tomboy
 		}
 	}
 
-#if ENABLE_GTKSPELL
-	public class NoteSpellChecker : NotePlugin
+#if FIXED_GTKSPELL
+	public class NoteSpellChecker : NoteAddin
 	{
 		IntPtr obj_ptr = IntPtr.Zero;
 
+		static bool gtkspell_available_tested;
+		static bool gtkspell_available_result;
+
 		[DllImport ("libgtkspell")]
 		static extern IntPtr gtkspell_new_attach (IntPtr text_view, 
 							  string locale, 
@@ -199,17 +202,50 @@ namespace Tomboy
 		[DllImport ("libgtkspell")]
 		static extern void gtkspell_detach (IntPtr obj);
 
-		protected override void Initialize ()
+		static bool DetectGtkSpellAvailable()
+		{
+			try {
+				Gtk.TextView test_view = new Gtk.TextView ();
+				IntPtr test_ptr = gtkspell_new_attach (test_view.Handle, 
+								       null, 
+								       IntPtr.Zero);
+				if (test_ptr != IntPtr.Zero)
+					gtkspell_detach (test_ptr);
+				return true;
+			} catch {
+				return false;
+			}
+		}
+
+		public static bool GtkSpellAvailable 
+		{
+			get {
+				if (!gtkspell_available_tested) {
+					gtkspell_available_result = DetectGtkSpellAvailable ();
+					gtkspell_available_tested = true;
+				}
+				return gtkspell_available_result;
+			}
+		}
+
+		public NoteSpellChecker ()
+		{
+			if (!GtkSpellAvailable) {
+				throw new Exception();
+			}
+		}
+
+		public override void Initialize ()
 		{
 			// Do nothing.
 		}
 
-		protected override void Shutdown ()
+		public override void Shutdown ()
 		{
 			// Do nothing.
 		}
 
-		protected override void OnNoteOpened ()
+		public override void OnNoteOpened ()
 		{
 			Buffer.TagApplied += TagApplied;
 			Preferences.SettingChanged += OnEnableSpellcheckChanged;
@@ -282,9 +318,9 @@ namespace Tomboy
 			}
 		}
 	}
-#endif // ENABLE_GTKSPELL
+#endif
 
-	public class NoteUrlWatcher : NotePlugin
+	public class NoteUrlWatcher : NoteAddin
 	{
 		NoteTag url_tag;
 		Gtk.TextMark click_mark;
@@ -302,17 +338,17 @@ namespace Tomboy
 			text_event_connected = false;
 		}
 
-		protected override void Initialize ()
+		public override void Initialize ()
 		{
 			url_tag = (NoteTag) Note.TagTable.Lookup ("link:url");
 		}
 
-		protected override void Shutdown ()
+		public override void Shutdown ()
 		{
 			// Do nothing.
 		}
 
-		protected override void OnNoteOpened ()
+		public override void OnNoteOpened ()
 		{
 #if FIXED_GTKSPELL
 			// NOTE: This hack helps avoid multiple URL opens for
@@ -524,7 +560,7 @@ namespace Tomboy
 		}
 	}
 
-	public class NoteLinkWatcher : NotePlugin
+	public class NoteLinkWatcher : NoteAddin
 	{
 		NoteTag url_tag;
 		NoteTag link_tag;
@@ -532,7 +568,7 @@ namespace Tomboy
 
 		static bool text_event_connected;
 
-		protected override void Initialize () 
+		public override void Initialize () 
 		{
 			Manager.NoteDeleted += OnNoteDeleted;
 			Manager.NoteAdded += OnNoteAdded;
@@ -543,14 +579,14 @@ namespace Tomboy
 			broken_link_tag = (NoteTag) Note.TagTable.Lookup ("link:broken");
 		}
 
-		protected override void Shutdown ()
+		public override void Shutdown ()
 		{
 			Manager.NoteDeleted -= OnNoteDeleted;
 			Manager.NoteAdded -= OnNoteAdded;
 			Manager.NoteRenamed -= OnNoteRenamed;
 		}
 
-		protected override void OnNoteOpened ()
+		public override void OnNoteOpened ()
 		{
 #if FIXED_GTKSPELL
 			// NOTE: This avoid multiple link opens for cases where
@@ -634,7 +670,7 @@ namespace Tomboy
 				if (range.Text.ToLower () != old_title_lower)
 					continue;
 
-				Logger.Log ("Replacing '{0}' with '{0}'", 
+				Logger.Log ("Replacing '{0}' with '{1}'", 
 					    range.Text, 
 					    renamed.Title);
 
@@ -749,12 +785,18 @@ namespace Tomboy
 				Logger.Log ("Creating note '{0}'...", link_name);
 				try {
 				    link = Manager.Create (link_name);
-				} catch (Exception e) {
+				} catch {
 				    // Fail silently.
 				}
 			}
-
-			if (link != null && link != this.Note) {
+			
+			// FIXME: We used to also check here for (link != this.Note), but
+			// somehow this was causing problems receiving clicks for the
+			// wrong instance of a note (see bug #413234).  Since a
+			// link:internal tag is never applied around text that's the same
+			// as the current note's title, it's safe to omit this check and
+			// also works around the bug.
+			if (link != null) {
 				Logger.Log ("Opening note '{0}' on click...", link_name);
 				link.Window.Present ();
 				return true;
@@ -772,7 +814,7 @@ namespace Tomboy
 		}
 	}
 
-	public class NoteWikiWatcher : NotePlugin
+	public class NoteWikiWatcher : NoteAddin
 	{
 		Gtk.TextTag broken_link_tag;
 
@@ -786,17 +828,17 @@ namespace Tomboy
 			regex = new Regex (WIKIWORD_REGEX, RegexOptions.Compiled);
 		}
 
-		protected override void Initialize ()
+		public override void Initialize ()
 		{
 			broken_link_tag = Note.TagTable.Lookup ("link:broken");
 		}
 
-		protected override void Shutdown ()
+		public override void Shutdown ()
 		{
 			// Do nothing.
 		}
 
-		protected override void OnNoteOpened ()
+		public override void OnNoteOpened ()
 		{
 			if ((bool) Preferences.Get (Preferences.ENABLE_WIKIWORDS)) {
 				Buffer.InsertText += OnInsertText;
@@ -880,7 +922,7 @@ namespace Tomboy
 		}
 	}
 
-	public class MouseHandWatcher : NotePlugin
+	public class MouseHandWatcher : NoteAddin
 	{
 		bool hovering_on_link;
 
@@ -893,17 +935,17 @@ namespace Tomboy
 			hand_cursor = new Gdk.Cursor (Gdk.CursorType.Hand2);
 		}
 
-		protected override void Initialize ()
+		public override void Initialize ()
 		{
 			// Do nothing.
 		}
 
-		protected override void Shutdown ()
+		public override void Shutdown ()
 		{
 			// Do nothing.
 		}
 
-		protected override void OnNoteOpened ()
+		public override void OnNoteOpened ()
 		{
 			Gtk.TextView editor = Window.Editor;
 			editor.MotionNotifyEvent += OnEditorMotion;
@@ -1010,4 +1052,56 @@ namespace Tomboy
 			}
 		}
 	}
+	
+	public class NoteTagsWatcher : NoteAddin
+	{
+		static NoteTagsWatcher ()
+		{
+		}
+
+		public override void Initialize ()
+		{
+			Note.TagAdded += OnTagAdded;
+			Note.TagRemoving += OnTagRemoving;
+			Note.TagRemoved += OnTagRemoved;
+		}
+
+		public override void Shutdown ()
+		{
+			Note.TagAdded -= OnTagAdded;
+			Note.TagRemoving -= OnTagRemoving;
+			Note.TagRemoved -= OnTagRemoved;
+		}
+
+		public override void OnNoteOpened ()
+		{
+			// FIXME: Just for kicks, spit out the current tags
+			Logger.Debug ("{0} tags:", Note.Title);
+			foreach (Tag tag in Note.Tags) {
+				Logger.Debug ("\t{0}", tag.Name);
+			}
+		}
+		
+		void OnTagAdded (Note note, Tag tag)
+		{
+			Logger.Debug ("Tag added to {0}: {1}", note.Title, tag.Name);
+		}
+		
+		void OnTagRemoving (Note note, Tag tag)
+		{
+			Logger.Debug ("Removing tag from {0}: {1}", note.Title, tag.Name);
+		}
+		
+		// <summary>
+		// Keep the TagManager clean by removing tags that are no longer
+		// tagging any other notes.
+		// </summary>
+		void OnTagRemoved (Note note, string tag_name)
+		{
+			Tag tag = TagManager.GetTag (tag_name);
+Logger.Debug ("Watchers.OnTagRemoved popularity count: {0}", tag.Popularity);
+			if (tag.Popularity == 0)
+				TagManager.RemoveTag (tag);
+		}
+	}
 }
